@@ -1,6 +1,6 @@
-/*  $Id$
+/*  $Id: prefs.c 6505 2009-01-19 18:59:42Z enrico $
  *
- *  Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *  Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "common.h"
 #include "prefs.h"
 #include "dictd.h"
+#include "spell.h"
 #include "wraplabel.h"
 
 
@@ -46,7 +47,24 @@ enum
 	NOTEBOOK_PAGE_GENERAL = 0,
 	NOTEBOOK_PAGE_DICTD,
 	NOTEBOOK_PAGE_WEB,
-	NOTEBOOK_PAGE_ASPELL
+	NOTEBOOK_PAGE_SPELL
+};
+
+static const web_dict_t web_dicts[] =
+{
+	{ N_("dict.leo.org - German <-> English"), "http://dict.leo.org/ende?search={word}" },
+	{ N_("dict.leo.org - German <-> French"), "http://dict.leo.org/frde?search={word}" },
+	{ N_("dict.leo.org - German <-> Spanish"), "http://dict.leo.org/esde?search={word}" },
+	{ N_("dict.leo.org - German <-> Italian"), "http://dict.leo.org/itde?search={word}" },
+	{ N_("dict.leo.org - German <-> Chinese"), "http://dict.leo.org/chde?search={word}" },
+	{ N_("dist.cc - Dictionary"), "http://www.dict.cc/?s={word}" },
+	{ N_("Dictionary.com"), "http://dictionary.reference.com/search?db=dictionary&q={word}" },
+	{ N_("TheFreeDictionary.com"), "http://www.thefreedictionary.com/_/partner.aspx?Word={word}&Set=www&mode=w" },
+	{ N_("Wikipedia, the free encyclopedia (EN)"), "http://en.wikipedia.org/wiki/{word}" },
+	{ N_("Wiktionary, the free dictionary (EN)"), "http://en.wiktionary.org/wiki/{word}" },
+	{ N_("Merriam-Webster Online Dictionary"), "http://www.merriam-webster.com/dictionary/{word}" },
+	{ N_("Clear"), "" },
+	{ NULL, NULL }
 };
 
 static void show_panel_entry_toggled(GtkToggleButton *tb, DictData *dd)
@@ -70,39 +88,6 @@ static void search_method_changed(GtkRadioButton *radiobutton, DictData *dd)
 }
 
 
-static void get_spell_dictionaries(GtkWidget *spell_combo, DictData *dd)
-{
-	if (NZV(dd->spell_bin))
-	{
-		gchar *tmp = NULL, *cmd, *locale_cmd;
-
-		cmd = g_strconcat(dd->spell_bin, " dump dicts", NULL);
-		locale_cmd = g_locale_from_utf8(cmd, -1, NULL, NULL, NULL);
-		if (locale_cmd == NULL)
-			locale_cmd = g_strdup(cmd);
-		g_spawn_command_line_sync(locale_cmd, &tmp, NULL, NULL, NULL);
-		if (NZV(tmp))
-		{
-			gchar **list = g_strsplit_set(tmp, "\n\r", -1);
-			gchar *item;
-			guint i, len = g_strv_length(list);
-			for (i = 0; i < len; i++)
-			{
-				item = g_strstrip(list[i]);
-				gtk_combo_box_append_text(GTK_COMBO_BOX(spell_combo), item);
-				if (strcmp(dd->spell_dictionary, item) == 0)
-					gtk_combo_box_set_active(GTK_COMBO_BOX(spell_combo), i);
-			}
-			g_strfreev(list);
-		}
-
-		g_free(cmd);
-		g_free(locale_cmd);
-		g_free(tmp);
-	}
-}
-
-
 void dict_prefs_dialog_response(GtkWidget *dlg, gint response, DictData *dd)
 {
 	gchar *dictionary, *search_url;
@@ -168,6 +153,9 @@ void dict_prefs_dialog_response(GtkWidget *dlg, gint response, DictData *dd)
 		dd->panel_entry_size = gtk_spin_button_get_value_as_int(
 					GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(dlg), "panel_entry_size_spinner")));
 	}
+	g_object_set(G_OBJECT(dd->link_tag), "foreground-gdk", dd->link_color, NULL);
+	g_object_set(G_OBJECT(dd->phon_tag), "foreground-gdk", dd->phon_color, NULL);
+
 	/* save settings */
 	dict_write_rc_file(dd);
 
@@ -190,20 +178,6 @@ static GtkWidget *create_web_dicts_table(GtkWidget *entry)
 	gint i;
 	gint offset;
 	GtkWidget *table, *button;
-	static web_dict_t web_dicts[] =
-	{
-		{ N_("dict.leo.org - German <-> English"), "http://dict.leo.org/ende?search={word}" },
-		{ N_("dict.leo.org - German <-> French"), "http://dict.leo.org/frde?search={word}" },
-		{ N_("dict.leo.org - German <-> Spanish"), "http://dict.leo.org/esde?search={word}" },
-		{ N_("dict.leo.org - German <-> Italian"), "http://dict.leo.org/itde?search={word}" },
-		{ N_("dict.leo.org - German <-> Chinese"), "http://dict.leo.org/chde?search={word}" },
-		{ N_("dist.cc - Dictionary"), "http://www.dict.cc/?s={word}" },
-		{ N_("Dictionary.com"), "http://dictionary.reference.com/search?db=dictionary&q={word}" },
-		{ N_("TheFreeDictionary.com"), "http://www.thefreedictionary.com/_/partner.aspx?Word={word}&Set=www&mode=w" },
-		{ N_("Wiktionary, the free dictionary (EN)"), "http://en.wiktionary.org/wiki/{word}" },
-		{ N_("Clear"), "" },
-		{ NULL, NULL }
-	};
 
 	table = gtk_table_new(4, 2, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
@@ -227,20 +201,87 @@ static GtkWidget *create_web_dicts_table(GtkWidget *entry)
 }
 
 
+const gchar *dict_prefs_get_web_url_label(DictData *dd)
+{
+	gint i;
+
+	for (i = 0; web_dicts[i].label != NULL; i++)
+	{
+		if (strcmp(web_dicts[i].url, dd->web_url) == 0)
+			return web_dicts[i].label;
+	}
+	return dd->web_url;
+}
+
+
+static void color_set_cb(GtkColorButton *widget, GdkColor *color)
+{
+	gtk_color_button_get_color(widget, color);
+}
+
+
+static void button_dict_refresh_cb(GtkWidget *button, DictData *dd)
+{
+	GtkWidget *combo = g_object_get_data(G_OBJECT(button), "spell_combo");
+
+	dict_spell_get_dictionaries(dd, combo);
+}
+
+
+static gboolean spell_entry_focus_cb(GtkEntry *entry, GdkEventFocus *ev, GtkWidget *icon)
+{
+	gchar *path = g_find_program_in_path(gtk_entry_get_text(entry));
+
+	if (path != NULL)
+	{
+		gtk_image_set_from_stock(GTK_IMAGE(icon), GTK_STOCK_YES, GTK_ICON_SIZE_BUTTON);
+		g_free(path);
+	}
+	else
+		gtk_image_set_from_stock(GTK_IMAGE(icon), GTK_STOCK_STOP, GTK_ICON_SIZE_BUTTON);
+
+	return FALSE;
+}
+
+
+static void spell_entry_activate_cb(GtkEntry *entry, DictData *dd)
+{
+	GtkWidget *combo = g_object_get_data(G_OBJECT(entry), "spell_combo");
+	GtkWidget *icon = g_object_get_data(G_OBJECT(entry), "icon");
+
+	spell_entry_focus_cb(entry, NULL, icon);
+	dict_spell_get_dictionaries(dd, combo);
+}
+
+
+static void spell_combo_changed_cb(GtkComboBox *widget, DictData *dd)
+{
+	GtkTreeIter iter;
+
+	if (gtk_combo_box_get_active_iter(widget, &iter))
+	{
+		gchar *text;
+		gtk_tree_model_get(gtk_combo_box_get_model(widget), &iter, 0, &text, -1);
+		g_free(dd->spell_dictionary);
+		dd->spell_dictionary = text;
+	}
+}
+
+
 GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 {
 	GtkWidget *dialog, *inner_vbox, *notebook, *notebook_vbox;
 	GtkWidget *label1, *label2, *label3;
 
 	dialog = xfce_titled_dialog_new_with_buttons(
-		_("Xfce4 Dictionary"), GTK_WINDOW(parent),
+		_("Dictionary"), GTK_WINDOW(parent),
 		GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
 		GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 		NULL);
 
 	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
-	gtk_window_set_icon_name(GTK_WINDOW(dialog), "xfce4-settings");
+	gtk_window_set_icon_name(GTK_WINDOW(dialog), "xfce4-dict");
 	if (! dd->is_plugin) /* the response callback is run by the plugin's callback */
 		g_signal_connect(dialog, "response", G_CALLBACK(dict_prefs_dialog_response), dd);
 
@@ -256,7 +297,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 	 */
 #define PAGE_GENERAL /* only for navigation in Geany's symbol list ;-) */
 	{
-		GtkWidget *radio_button, *label;
+		GtkWidget *radio_button, *label, *table, *color_link, *color_phon;
 		GSList *search_method;
 
 		notebook_vbox = gtk_vbox_new(FALSE, 2);
@@ -273,7 +314,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		gtk_widget_show(label);
 		gtk_box_pack_start(GTK_BOX(inner_vbox), label, FALSE, FALSE, 0);
 
-		radio_button = gtk_radio_button_new_with_label(NULL, _("Dict"));
+		radio_button = gtk_radio_button_new_with_label(NULL, _("Dictionary Server"));
 		search_method = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio_button));
 		if (dd->mode_default == DICTMODE_DICT)
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button), TRUE);
@@ -282,7 +323,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		g_object_set_data(G_OBJECT(radio_button), "type", GINT_TO_POINTER(DICTMODE_DICT));
 		g_signal_connect(G_OBJECT(radio_button), "toggled", G_CALLBACK(search_method_changed), dd);
 
-		radio_button = gtk_radio_button_new_with_label(search_method, _("Web"));
+		radio_button = gtk_radio_button_new_with_label(search_method, _("Web Service"));
 		search_method = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio_button));
 		if (dd->mode_default == DICTMODE_WEB)
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button), TRUE);
@@ -291,7 +332,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		g_object_set_data(G_OBJECT(radio_button), "type", GINT_TO_POINTER(DICTMODE_WEB));
 		g_signal_connect(G_OBJECT(radio_button), "toggled", G_CALLBACK(search_method_changed), dd);
 
-		radio_button = gtk_radio_button_new_with_label(search_method, _("Spellcheck"));
+		radio_button = gtk_radio_button_new_with_label(search_method, _("Spell Checker"));
 		search_method = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radio_button));
 		if (dd->mode_default == DICTMODE_SPELL)
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button), TRUE);
@@ -309,12 +350,52 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		g_object_set_data(G_OBJECT(radio_button), "type", GINT_TO_POINTER(DICTMODE_LAST_USED));
 		g_signal_connect(G_OBJECT(radio_button), "toggled", G_CALLBACK(search_method_changed), dd);
 
+		label = gtk_label_new(_("<b>Colors:</b>"));
+		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+		gtk_misc_set_alignment(GTK_MISC(label), 0, 1);
+		gtk_widget_show(label);
+		gtk_box_pack_start(GTK_BOX(inner_vbox), label, FALSE, FALSE, 5);
+
+		label1 = gtk_label_new(_("Link Color:"));
+		label2 = gtk_label_new(_("Phonetic Color:"));
+		color_link = gtk_color_button_new_with_color(dd->link_color);
+		color_phon = gtk_color_button_new_with_color(dd->phon_color);
+		g_signal_connect(color_link, "color-set", G_CALLBACK(color_set_cb), dd->link_color);
+		g_signal_connect(color_phon, "color-set", G_CALLBACK(color_set_cb), dd->phon_color);
+
+		table = gtk_table_new(2, 2, FALSE);
+		gtk_widget_show(table);
+		gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+		gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+
+		gtk_table_attach(GTK_TABLE(table), label1, 0, 1, 0, 1,
+						(GtkAttachOptions) (GTK_FILL),
+						(GtkAttachOptions) (0), 5, 5);
+		gtk_misc_set_alignment(GTK_MISC(label1), 1, 0);
+
+		gtk_table_attach(GTK_TABLE(table), color_link, 1, 2, 0, 1,
+						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
+						(GtkAttachOptions) (0), 5, 5);
+
+		gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 1, 2,
+						(GtkAttachOptions) (GTK_FILL),
+						(GtkAttachOptions) (0), 5, 0);
+		gtk_misc_set_alignment(GTK_MISC(label2), 1, 0);
+
+		gtk_table_attach(GTK_TABLE(table), color_phon, 1, 2, 1, 2,
+						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
+						(GtkAttachOptions) (0), 5, 5);
+
+		gtk_widget_show_all(table);
+		gtk_box_pack_start(GTK_BOX(inner_vbox), table, FALSE, FALSE, 0);
+
+
 		/* show panel entry check box */
 		if (dd->is_plugin)
 		{
 			GtkWidget *pe_hbox, *panel_entry_size_label, *panel_entry_size_spinner, *check_panel_entry;
 
-			label = gtk_label_new(_("<b>Panel text field:</b>"));
+			label = gtk_label_new(_("<b>Panel Text Field:</b>"));
 			gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 			gtk_misc_set_alignment(GTK_MISC(label), 0, 1);
 			gtk_widget_show(label);
@@ -361,7 +442,8 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 	 */
 #define PAGE_DICTD /* only for navigation in Geany's symbol list ;-) */
 	 {
-		GtkWidget *table, *button_get_list, *server_entry, *port_spinner, *dict_combo;
+		GtkWidget *table, *button_get_list, *button_get_info;
+		GtkWidget *server_entry, *port_spinner, *dict_combo;
 
 		notebook_vbox = gtk_vbox_new(FALSE, 2);
 		gtk_widget_show(notebook_vbox);
@@ -369,7 +451,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		gtk_container_set_border_width(GTK_CONTAINER(inner_vbox), 5);
 		gtk_widget_show(inner_vbox);
 		gtk_notebook_insert_page(GTK_NOTEBOOK(notebook),
-			notebook_vbox, gtk_label_new(_("Dict")), NOTEBOOK_PAGE_DICTD);
+			notebook_vbox, gtk_label_new(_("Dictionary Server")), NOTEBOOK_PAGE_DICTD);
 
 		/* server address */
 		label1 = gtk_label_new_with_mnemonic(_("Server:"));
@@ -419,13 +501,19 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		g_object_set_data(G_OBJECT(dialog), "port_spinner", port_spinner);
 		g_object_set_data(G_OBJECT(dialog), "dict_combo", dict_combo);
 
-		button_get_list = gtk_button_new_from_stock("gtk-find");
+		button_get_list = gtk_button_new_from_stock("gtk-refresh");
 		gtk_widget_show(button_get_list);
 		g_signal_connect(button_get_list, "clicked", G_CALLBACK(dict_dictd_get_list), dd);
 		g_object_set_data(G_OBJECT(button_get_list), "dict_combo", dict_combo);
 		g_object_set_data(G_OBJECT(button_get_list), "port_spinner", port_spinner);
 		g_object_set_data(G_OBJECT(button_get_list), "server_entry", server_entry);
 
+		button_get_info = gtk_button_new_from_stock("gtk-info");
+		gtk_widget_show(button_get_info);
+		g_signal_connect(button_get_info, "clicked", G_CALLBACK(dict_dictd_get_information), dd);
+		g_object_set_data(G_OBJECT(button_get_info), "port_spinner", port_spinner);
+		g_object_set_data(G_OBJECT(button_get_info), "server_entry", server_entry);
+
 		/* put it all together */
 		table = gtk_table_new(3, 3, FALSE);
 		gtk_widget_show(table);
@@ -441,6 +529,10 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
 						(GtkAttachOptions) (0), 5, 5);
 
+		gtk_table_attach(GTK_TABLE(table), button_get_info, 2, 3, 0, 1,
+						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
+						(GtkAttachOptions) (0), 5, 5);
+
 		gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 1, 2,
 						(GtkAttachOptions) (GTK_FILL),
 						(GtkAttachOptions) (0), 5, 0);
@@ -480,7 +572,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		gtk_container_set_border_width(GTK_CONTAINER(inner_vbox), 5);
 		gtk_widget_show(inner_vbox);
 		gtk_notebook_insert_page(GTK_NOTEBOOK(notebook),
-			notebook_vbox, gtk_label_new(_("Web")), NOTEBOOK_PAGE_WEB);
+			notebook_vbox, gtk_label_new(_("Web Service")), NOTEBOOK_PAGE_WEB);
 
 		label = gtk_label_new(_("<b>Web search URL:</b>"));
 		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
@@ -508,7 +600,7 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 
 		g_object_set_data(G_OBJECT(dialog), "web_entry", web_entry);
 
-		label1 = wrap_label_new(_("Enter an URL to a web site which offer translation or dictionary services. Use {word} as placeholder for the searched word."));
+		label1 = xfd_wrap_label_new(_("Enter an URL to a web site which offer translation or dictionary services. Use {word} as placeholder for the searched word."));
 		gtk_misc_set_alignment(GTK_MISC(label1), 0, 0);
 		gtk_widget_show(label1);
 		gtk_box_pack_start(GTK_BOX(inner_vbox), label1, FALSE, FALSE, 0);
@@ -516,11 +608,13 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 	}
 
 	/*
-	 * Page: ASPELL
+	 * Page: SPELL
 	 */
-#define PAGE_ASPELL /* only for navigation in Geany's symbol list ;-) */
+#define PAGE_SPELL /* only for navigation in Geany's symbol list ;-) */
 	{
-		GtkWidget *table, *spell_entry, *spell_combo;
+		GtkWidget *table, *label_help, *spell_entry, *spell_combo, *button_refresh, *image, *icon;
+		GtkListStore *store;
+		GtkCellRenderer *renderer;
 
 		notebook_vbox = gtk_vbox_new(FALSE, 5);
 		gtk_widget_show(notebook_vbox);
@@ -528,53 +622,96 @@ GtkWidget *dict_prefs_dialog_show(GtkWidget *parent, DictData *dd)
 		gtk_container_set_border_width(GTK_CONTAINER(inner_vbox), 5);
 		gtk_widget_show(inner_vbox);
 		gtk_notebook_insert_page(GTK_NOTEBOOK(notebook),
-			notebook_vbox, gtk_label_new(_("Aspell")), NOTEBOOK_PAGE_ASPELL);
+			notebook_vbox, gtk_label_new(_("Spell Checker")), NOTEBOOK_PAGE_SPELL);
 
-		label1 = gtk_label_new_with_mnemonic(_("Aspell program:"));
+		label1 = gtk_label_new_with_mnemonic(_("Spell Check Program:"));
 		gtk_widget_show(label1);
 
+		icon = gtk_image_new();
+		gtk_widget_show(icon);
+
 		spell_entry = gtk_entry_new();
 		gtk_entry_set_max_length(GTK_ENTRY(spell_entry), 256);
 		if (dd->spell_bin != NULL)
 		{
 			gtk_entry_set_text(GTK_ENTRY(spell_entry), dd->spell_bin);
 		}
+		g_signal_connect(spell_entry, "focus-out-event", G_CALLBACK(spell_entry_focus_cb), icon);
+		g_signal_connect(spell_entry, "activate", G_CALLBACK(spell_entry_activate_cb), dd);
 		gtk_widget_show(spell_entry);
 
+		label_help = xfd_wrap_label_new(_(
+	"<i>The spell check program can be 'enchant', 'aspell', 'ispell' or any other spell check "
+	"program which is compatible to the ispell command.\nThe icon shows whether the entered "
+	"command exists.</i>"));
+		gtk_label_set_use_markup(GTK_LABEL(label_help), TRUE);
+		gtk_widget_show(label_help);
+
 		label2 = gtk_label_new_with_mnemonic(_("Dictionary:"));
 		gtk_widget_show(label2);
 
-		spell_combo = gtk_combo_box_new_text();
-		get_spell_dictionaries(spell_combo, dd);
+		store = gtk_list_store_new(1, G_TYPE_STRING);
+		spell_combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
+		g_object_set_data(G_OBJECT(spell_combo), "spell_entry", spell_entry);
+
+		renderer = gtk_cell_renderer_text_new();
+		gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(spell_combo), renderer, TRUE);
+		gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(spell_combo), renderer, "text", 0);
+
+		dict_spell_get_dictionaries(dd, spell_combo);
+		g_signal_connect(spell_combo, "changed", G_CALLBACK(spell_combo_changed_cb), dd);
 		gtk_widget_show(spell_combo);
 
+		button_refresh = gtk_button_new();
+		image = gtk_image_new_from_stock("gtk-refresh", GTK_ICON_SIZE_BUTTON);
+		gtk_button_set_image(GTK_BUTTON(button_refresh), image);
+		gtk_widget_show(button_refresh);
+		g_object_set_data(G_OBJECT(button_refresh), "spell_combo", spell_combo);
+		g_signal_connect(button_refresh, "clicked", G_CALLBACK(button_dict_refresh_cb), dd);
+
+		g_object_set_data(G_OBJECT(spell_entry), "icon", icon);
+		g_object_set_data(G_OBJECT(spell_entry), "spell_combo", spell_combo);
 		g_object_set_data(G_OBJECT(dialog), "spell_combo", spell_combo);
 		g_object_set_data(G_OBJECT(dialog), "spell_entry", spell_entry);
+		g_object_unref(store);
 
+		spell_entry_focus_cb(GTK_ENTRY(spell_entry), NULL, icon); /* initially set the icon */
 
-		table = gtk_table_new(2, 2, FALSE);
+		table = gtk_table_new(3, 3, FALSE);
 		gtk_widget_show(table);
 		gtk_table_set_row_spacings(GTK_TABLE(table), 5);
 		gtk_table_set_col_spacings(GTK_TABLE(table), 5);
 
-		gtk_table_attach(GTK_TABLE(table), label1, 0, 1, 0, 1,
+		gtk_table_attach(GTK_TABLE(table), label_help, 0, 3, 0, 1,
+						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
+						(GtkAttachOptions) (0), 5, 5);
+
+		gtk_table_attach(GTK_TABLE(table), label1, 0, 1, 1, 2,
 						(GtkAttachOptions) (GTK_FILL),
 						(GtkAttachOptions) (0), 5, 5);
 		gtk_misc_set_alignment(GTK_MISC(label1), 1, 0);
 
-		gtk_table_attach(GTK_TABLE(table), spell_entry, 1, 2, 0, 1,
+		gtk_table_attach(GTK_TABLE(table), spell_entry, 1, 2, 1, 2,
 						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
 						(GtkAttachOptions) (0), 5, 5);
 
-		gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 1, 2,
+		gtk_table_attach(GTK_TABLE(table), icon, 2, 3, 1, 2,
+						(GtkAttachOptions) (0),
+						(GtkAttachOptions) (0), 5, 5);
+
+		gtk_table_attach(GTK_TABLE(table), label2, 0, 1, 2, 3,
 						(GtkAttachOptions) (GTK_FILL),
 						(GtkAttachOptions) (0), 5, 0);
 		gtk_misc_set_alignment(GTK_MISC(label2), 1, 0);
 
-		gtk_table_attach(GTK_TABLE(table), spell_combo, 1, 2, 1, 2,
+		gtk_table_attach(GTK_TABLE(table), spell_combo, 1, 2, 2, 3,
 						(GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
 						(GtkAttachOptions) (0), 5, 5);
 
+		gtk_table_attach(GTK_TABLE(table), button_refresh, 2, 3, 2, 3,
+						(GtkAttachOptions) (0),
+						(GtkAttachOptions) (0), 5, 5);
+
 		gtk_box_pack_start(GTK_BOX(inner_vbox), table, FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(notebook_vbox), inner_vbox, TRUE, TRUE, 5);
 	}
